% A simple demo of one QWLD feature extraction: 
% QUATERNIONIC INCREMENT BASED WEBER DESCRIPTOR (QIWD)
% proposed in IEEE TCSVT 2016.
% rushi lan (rslan2012@gmail.com); 2016-05-09

clear, clc, close all

im = imread('exam5.png'); % im = imresize(im,[128 64]);
figure, imshow(im);

%%% two parameters used in QIWD
alfa1 = 0.75;
alfa2 = 1.25;


im = double(im);

[m n colordim] = size(im);

lbp_im_m = zeros(m, n);
lbp_im_I = zeros(m, n);
lbp_im_J = zeros(m, n);
lbp_im_K = zeros(m, n);

eim = padarray(im, [1 1], 'symmetric');

mu = quaternion(1,1,1)/sqrt(3);

im = double(im)./255;  eim = double(eim)./255;
[mm nn cd] = size(eim);
im = quaternion(im(:,:,1), im(:,:,2), im(:,:,3));
eim = quaternion(eim(:,:,1), eim(:,:,2), eim(:,:,3));

% abs_eim = abs(eim);

filter1 = [1 1 1; 1 -8 1;1 1 1];

% alfa1 = 0.75;
% alfa2 = 1.25;

resu_conv = conv2(im,filter1);
resu_conv = resu_conv(2:(m+1),2:(n+1));
resu_temp_0 = resu_conv./(im + eps);

resu_temp = resu_temp_0 + 8*(1-alfa1);
resu_de_1 = angle(resu_temp);
lbp_im_m = uint8(255*resu_de_1./pi);% figure,imshow(lbp_im_m);
% f1 = imhist(lbp_im_m(2:(m-1),2:(n-1))); %  figure,plot(f1);

resu_temp = resu_temp_0 + 8*(1-alfa2);
resu_de_1 = angle(resu_temp);
lbp_im_I = uint8(255*resu_de_1./pi);%figure,imshow(lbp_im_m);
% f2 = imhist(lbp_im_m(2:(m-1),2:(n-1)));  % figure,plot(f2);


filter2 = [0 0 0;1 0 -1;0 0 0];
filter3 = [0 -1 0; 0 0 0; 0 1 0];

resu_conv_1 = conv2(im,filter2);resu_conv_1 = resu_conv_1(2:(m+1),2:(n+1));
resu_conv_2 = conv2(im,filter3);resu_conv_2 = resu_conv_2(2:(m+1),2:(n+1));
O_1 = angle(resu_conv_1./(resu_conv_2+eps));

lbp_im_J = uint8(255*O_1./pi);% figure,imshow(lbp_im_m);
% f3 = imhist(lbp_im_m(2:(m-1),2:(n-1)));   % figure,plot(f3);


filter2 = [0 0 -1;0 0 0;1 0 0];
filter3 = [-1 0 0; 0 0 0; 0 0 1];

resu_conv_1 = conv2(im,filter2);resu_conv_1 = resu_conv_1(2:(m+1),2:(n+1));
resu_conv_2 = conv2(im,filter3);resu_conv_2 = resu_conv_2(2:(m+1),2:(n+1));
O_1 = angle(resu_conv_1./(resu_conv_2+eps));

lbp_im_K = uint8(255*O_1./pi);%figure,imshow(lbp_im_m);
% f4 = imhist(lbp_im_m(2:(m-1),2:(n-1)));  % figure,plot(f4);


lbp_im = zeros(m,n,4);

lbp_im(:,:,1) = lbp_im_m;
lbp_im(:,:,2) = lbp_im_I;
lbp_im(:,:,3) = lbp_im_J;
lbp_im(:,:,4) = lbp_im_K;

colordim = 4;

%%% divide the image into several dense local patches, and use the
%%% histogram of each patch as features;
%%% for images with different size, you need to change the patch numbers.

xp = round(linspace(5, 124, 12));
yp = round(linspace(5, 60, 6));

f = [];
[m n mn] = size(lbp_im);
r = 8;   %%% patch size: (2*r+1)*(2*r+1)
for cd = 1:colordim
    for i = 1:length(xp)
        for j=1:length(yp)
            x = xp(i);
            y = yp(j);
            xmin = max(x-r, 1); xmax = min(x+r, m);
            ymin = max(y-r, 1); ymax = min(y+r, n);
            temp = lbp_im(xmin:xmax, ymin:ymax, cd);   % local patch in the image
            f0 = imhist(uint8(temp),16);      % bin number of histogram 
            f0 = f0/sum(f0);
            f = [f; f0];
        end
    end
end
figure, plot(f);   % final feature used for classification